@@ -5,12 +5,12 @@ import math
 import random
 import time
 import typing
-import warnings
 
 import requests
 
 from .models import MAX_ROWS_PER_PAGE
 
+logger = logging.getLogger(__name__)
 
 def _random_pause():
     """sleep for at least .333 seconds"""
@@ -100,6 +100,7 @@ def _request(
     url: str,
     headers: dict,
     timeout: int = 30,
+    max_attempts: int = 5,
     params: dict = None,
     data: dict = None,
     files: BufferedReader = None,
@@ -109,18 +110,42 @@ def _request(
         method, url, headers=headers, params=params, json=data, files=files
     )
     prepped = req.prepare()
-    res = session.send(prepped, timeout=timeout)
-    res.raise_for_status()
+
+    attempts = 1
+
+    while True:
+        logger.debug(
+            f"{method} to {url} with {params or 'no params'} (Attempt {attempts}/{max_attempts})"  # noqa:E501
+        )
+
+        try:
+            res = session.send(prepped, timeout=timeout)
+            res.raise_for_status()
+
+        except (requests.exceptions.Timeout, requests.exceptions.HTTPError) as e:
+            """5xx errors (a recurring problem with the Knack API) and Timeouts
+            (both  ConnectTimeout and ReadTimeout) are suppresed based on
+            max_attempts. Any other error is raised"""
+            if e.response and e.response.status_code < 500:
+                raise e
+
+            if attempts < max_attempts:
+                logger.debug(f"Error on attempt #{attempts}: {e.__repr__()}")
+                attempts += 1
+                _random_pause()
+                continue
+            else:
+                raise e
+        break
     return res
 
 
-def _continue(total_records: int, current_records: int, record_limit: int) -> bool:
+def _continue(total_records: int, current_record_count: int, record_limit: int) -> bool:
     if total_records is None:
+        # this case only happens on the *first* API request
         return True
-
-    elif current_records < record_limit and total_records > current_records:
+    elif current_record_count < record_limit and total_records > current_record_count:
         return True
-
     return False
 
 
@@ -141,46 +166,26 @@ def _get_paginated_records(
     page = 1
 
     while _continue(total_records, len(records), record_limit):
-        attempts = 0
         params = {"page": page, "rows_per_page": rows_per_page, "filters": filters}
+        logger.debug(f"Getting {rows_per_page} records from page {page} from {url}")
+        res = _request(
+            method="GET",
+            url=url,
+            headers=headers,
+            timeout=timeout,
+            max_attempts=max_attempts,
+            params=params,
+        )
 
-        while True:
-            logging.debug(
-                f"Getting {rows_per_page} records from page {page} from {url}"
-            )
-
-            try:
-                res = _request(
-                    method="GET",
-                    url=url,
-                    headers=headers,
-                    timeout=timeout,
-                    params=params,
-                )
-
-            except (requests.exceptions.Timeout, requests.exceptions.HTTPError) as e:
-                """5xx errors (a recurring problem with the Knack API) and Timeouts
-                (both  ConnectTimeout and ReadTimeout) are suppresed based on
-                max_attempts. Any other error is raised"""
-                if e.response and e.response.status_code < 500:
-                    raise e
-
-                if attempts < max_attempts:
-                    warnings.warn(
-                        f"Error on attempt #{attempts}: {e.__repr__()}"
-                    )
-                    attempts += 1
-                    _random_pause()
-                    continue
-                else:
-                    raise e
+        fetched_records = res.json()["records"]        
+        if len(fetched_records) == 0:
+            """Failsafe to handle edge case in which Knack returns fewer records than expected from 
+            total_records. Consider `total_records` an estimate"""
             break
 
-        total_records = res.json()[
-            "total_records"
-        ]  # note that this number could change between requests
-        records += res.json()["records"]
+        records += fetched_records
         page += 1
+        total_records = res.json()["total_records"]
 
     # lazily shaving off any remainder to keep the client happy
     return records[0:record_limit] if record_limit < math.inf else records
@@ -221,17 +226,12 @@ def get(
         list: Knack records.
     """
     route = _route(obj=obj, scene=scene, view=view)
-
     url = _url(slug=slug, route=route)
-
     record_limit = record_limit if record_limit else math.inf
-
     filters = json.dumps(filters) if filters else None
-
     rows_per_page = (
         MAX_ROWS_PER_PAGE if record_limit >= MAX_ROWS_PER_PAGE else record_limit
     )
-
     return _get_paginated_records(
         app_id=app_id,
         api_key=api_key,
@@ -243,7 +243,9 @@ def get(
     )
 
 
-def get_metadata(*, app_id: str, slug: str = None, timeout: int = 30) -> dict:
+def get_metadata(
+    *, app_id: str, slug: str = None, timeout: int = 30, max_attempts: int = 5
+) -> dict:
     """Fetch Knack application metadata. You can find your app's metadata at:
     `https://api.knack.com/v1/applications/<app_id:str>`.
 
@@ -257,7 +259,9 @@ def get_metadata(*, app_id: str, slug: str = None, timeout: int = 30) -> dict:
     """
     route = _route(app_id=app_id)
     url = _url(slug=slug, route=route)
-    return _request(method="GET", url=url, headers=None).json()
+    return _request(
+        method="GET", url=url, headers=None, max_attempts=max_attempts
+    ).json()
 
 
 def _handle_method(method: str):
@@ -306,14 +310,22 @@ def record(
             [Requests docs](https://requests.readthedocs.io/en/master/user/quickstart/).
 
     Returns:
-        dict: The updated or newly created Knack record data, or, if deleting a record: `{"delete": true}`
+        dict: The updated or newly created Knack record data, or, if deleting a
+        record: `{"delete": true}`
     """
     record_id = data["id"] if method != "create" else ""
     headers = _headers(app_id, api_key)
     route = _route(obj=obj, record_id=record_id)
     method = _handle_method(method)
     url = _url(slug=slug, route=route)
-    return _request(method=method, url=url, headers=headers, data=data).json()
+    return _request(
+        method=method,
+        url=url,
+        headers=headers,
+        data=data,
+        max_attempts=max_attempts,
+        timeout=timeout,
+    ).json()
 
 
 def upload(
@@ -339,7 +351,7 @@ def upload(
     Args:
         app_id (str): Knack [application ID](https://www.knack.com/developer-documentation/#find-your-api-key-amp-application-id)  # noqa:E501
             string.
-        api_key (str): [Knack API key](https://www.knack.com/developer-documentation/#find-your-api-key-amp-application-id).
+        api_key (str): [Knack API key](https://www.knack.com/developer-documentation/#find-your-api-key-amp-application-id).  # noqa:E501
         obj (str): The Knack object key which holds the record data.
         field (str): The knack field key of the field you're uploading into.
         path (str): The path to the file to be uploaded.
